@@ -7,6 +7,8 @@
 #
 # See the file "license.terms" for information on usage and redistribution of
 # this file, and for a DISCLAIMER OF ALL WARRANTIES.
+#
+# RCS: @(#) $Id: http.tcl,v 1.60 2007/03/12 22:08:40 patthoyts Exp $
 
 # Rough version history:
 # 1.0	Old http_get interface.
@@ -22,7 +24,7 @@
 package require Tcl 8.4
 # Keep this in sync with pkgIndex.tcl and with the install directories
 # in Makefiles
-package provide http 2.5.8
+package provide http 2.5.3
 
 namespace eval http {
     variable http
@@ -44,11 +46,11 @@ namespace eval http {
 	for {set i 0} {$i <= 256} {incr i} {
 	    set c [format %c $i]
 	    if {![string match {[-._~a-zA-Z0-9]} $c]} {
-		set map($c) %[format %.2X $i]
+		set map($c) %[format %.2x $i]
 	    }
 	}
 	# These are handled specially
-	set map(\n) %0D%0A
+	set map(\n) %0d%0a
 	variable formMap [array get map]
     }
     init
@@ -62,8 +64,8 @@ namespace eval http {
     # This can be changed, but iso8859-1 is the RFC standard.
     variable defaultCharset "iso8859-1"
 
-    # Force RFC 3986 strictness in geturl url verification?  Not for 8.4.x
-    variable strict 0
+    # Force RFC 3986 strictness in geturl url verification?
+    variable strict 1
 
     namespace export geturl config reset wait formatQuery register unregister
     # Useful, but not exported: data size status code
@@ -346,7 +348,7 @@ proc http::geturl { url args } {
 	    ( [^/:\#?]+ )		# <host part of authority>
 	    (?: : (\d+) )?		# <port part of authority>
 	)?
-	( [/\?] [^\#]*)?		# <path> (including query)
+	( / [^\#?]* (?: \? [^\#?]* )?)?	# <path> (including query)
 	(?: \# (.*) )?			# <fragment>
 	$
     }
@@ -389,12 +391,6 @@ proc http::geturl { url args } {
 	}
     }
     if {$srvurl ne ""} {
-	# RFC 3986 allows empty paths (not even a /), but servers
-	# return 400 if the path in the HTTP request doesn't start
-	# with / , so add it here if needed.
-	if {[string index $srvurl 0] ne "/"} {
-	    set srvurl /$srvurl
-	}
 	# Check for validity according to RFC 3986, Appendix A
 	set validityRE {(?xi)
 	    ^
@@ -486,26 +482,19 @@ proc http::geturl { url args } {
 	fileevent $s writable [list http::Connect $token]
 	http::wait $token
 
-	if {![info exists state]} {
-	    # If we timed out then Finish has been called and the users
-	    # command callback may have cleaned up the token. If so
-	    # we end up here with nothing left to do.
+	if {$state(status) eq "error"} {
+	    # Something went wrong while trying to establish the connection.
+	    # Clean up after events and such, but DON'T call the command
+	    # callback (if available) because we're going to throw an
+	    # exception from here instead.
+	    set err [lindex $state(error) 0]
+	    cleanup $token
+	    return -code error $err
+	} elseif {$state(status) ne "connect"} {
+	    # Likely to be connection timeout
 	    return $token
-	} else {
-	    if {$state(status) eq "error"} {
-		# Something went wrong while trying to establish the connection.
-		# Clean up after events and such, but DON'T call the command
-		# callback (if available) because we're going to throw an
-		# exception from here instead.
-		set err [lindex $state(error) 0]
-		cleanup $token
-		return -code error $err
-	    } elseif {$state(status) ne "connect"} {
-		# Likely to be connection timeout
-		return $token
-	    }
-	    set state(status) ""
 	}
+	set state(status) ""
     }
 
     # Send data in cr-lf format, but accept any line terminators
@@ -621,7 +610,7 @@ proc http::geturl { url args } {
 
 	# if state(status) is error, it means someone's already called Finish
 	# to do the above-described clean up.
-	if {$state(status) ne "error"} {
+	if {$state(status) eq "error"} {
 	    Finish $token $err 1
 	}
 	cleanup $token
@@ -643,7 +632,6 @@ proc http::data {token} {
     return $state(body)
 }
 proc http::status {token} {
-    if {![info exists $token]} { return "error" }    
     variable $token
     upvar 0 $token state
     return $state(status)
@@ -667,11 +655,7 @@ proc http::size {token} {
     upvar 0 $token state
     return $state(currentsize)
 }
-proc http::meta {token} {
-    variable $token
-    upvar 0 $token state
-    return $state(meta)
-}
+
 proc http::error {token} {
     variable $token
     upvar 0 $token state
@@ -713,10 +697,10 @@ proc http::cleanup {token} {
 proc http::Connect {token} {
     variable $token
     upvar 0 $token state
-    set err "due to unexpected EOF"
+    global errorInfo errorCode
     if {[eof $state(sock)] ||
-	    [string length [set err [fconfigure $state(sock) -error]]]} {
-	Finish $token "connect failed $err" 1
+	[string length [fconfigure $state(sock) -error]]} {
+	    Finish $token "connect failed [fconfigure $state(sock) -error]" 1
     } else {
 	set state(status) connect
 	fileevent $state(sock) writable {}
@@ -802,9 +786,13 @@ proc http::Event {token} {
     upvar 0 $token state
     set s $state(sock)
 
+     if {[eof $s]} {
+	Eof $token
+	return
+    }
     if {$state(state) eq "header"} {
 	if {[catch {gets $s line} n]} {
-	    return [Finish $token $n]
+	    Finish $token $n
 	} elseif {$n == 0} {
 	    variable encodings
 	    set state(state) body
@@ -832,7 +820,6 @@ proc http::Event {token} {
 		# Initiate a sequence of background fcopies
 		fileevent $s readable {}
 		CopyStart $s $token
-		return
 	    }
 	} elseif {$n > 0} {
 	    if {[regexp -nocase {^content-type:(.+)$} $line x type]} {
@@ -867,7 +854,7 @@ proc http::Event {token} {
 		incr state(currentsize) $n
 	    }
 	} err]} {
-	    return [Finish $token $err]
+	    Finish $token $err
 	} else {
 	    if {[info exists state(-progress)]} {
 		eval $state(-progress) \
@@ -875,11 +862,6 @@ proc http::Event {token} {
 	    }
 	}
     }
-
-    if {[eof $s]} {
-	Eof $token
-	return
-    }
 }
 
 # http::CopyStart
@@ -975,7 +957,7 @@ proc http::wait {token} {
 	vwait $token\(status)
     }
 
-    return [status $token]
+    return $state(status)
 }
 
 # http::formatQuery --
@@ -1055,7 +1037,3 @@ proc http::ProxyRequired {host} {
 	return [list $http(-proxyhost) $http(-proxyport)]
     }
 }
-
-# Local variables:
-# indent-tabs-mode: t
-# End:
