@@ -5,16 +5,20 @@
 #
 #
 # Copyright (c) 1998-1999 by Scriptics Corporation.
+# Copyright (c) 2007 Daniel A. Steffen <das@users.sourceforge.net>
+#
 # See the file "license.terms" for information on usage and redistribution
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
+#
+# RCS: @(#) $Id: genStubs.tcl,v 1.22 2007/12/13 15:28:40 dgp Exp $
 
-package require Tcl 8
+package require Tcl 8.4
 
 namespace eval genStubs {
     # libraryName --
     #
     #	The name of the entire library.  This value is used to compute
-    #	the USE_*_STUBS macro and the name of the init file.
+    #	the USE_*_STUB_PROCS macro and the name of the init file.
 
     variable libraryName "UNKNOWN"
 
@@ -31,22 +35,6 @@ namespace eval genStubs {
 
     variable curName "UNKNOWN"
 
-    # scspec --
-    #
-    #	Storage class specifier for external function declarations.
-    #	Normally "EXTERN", may be set to something like XYZAPI
-    #
-    variable scspec "EXTERN"
-
-    # epoch, revision --
-    #
-    #	The epoch and revision numbers of the interface currently being defined.
-    #   (@@@TODO: should be an array mapping interface names -> numbers)
-    #
-
-    variable epoch {}
-    variable revision 0
-
     # hooks --
     #
     #	An array indexed by interface name that contains the set of
@@ -108,27 +96,6 @@ proc genStubs::interface {name} {
     return
 }
 
-# genStubs::scspec --
-#
-#	Define the storage class macro used for external function declarations.
-#	Typically, this will be a macro like XYZAPI or EXTERN that
-#	expands to either DLLIMPORT or DLLEXPORT, depending on whether
-#	-DBUILD_XYZ has been set.
-#
-proc genStubs::scspec {value} {
-    variable scspec $value
-}
-
-# genStubs::epoch --
-#
-#	Define the epoch number for this library.  The epoch
-#	should be incrememented when a release is made that
-#	contains incompatible changes to the public API.
-#
-proc genStubs::epoch {value} {
-    variable epoch $value
-}
-
 # genStubs::hooks --
 #
 #	This function defines the subinterface hooks for the current
@@ -167,18 +134,11 @@ proc genStubs::hooks {names} {
 proc genStubs::declare {args} {
     variable stubs
     variable curName
-    variable revision
 
-    incr revision
-    if {[llength $args] == 2} {
-	lassign $args index decl
-	set platformList generic
-    } elseif {[llength $args] == 3} {
-	lassign $args index platformList decl
-    } else {
+    if {[llength $args] != 3} {
 	puts stderr "wrong # args: declare $args"
-	return
     }
+    lassign $args index platformList decl
 
     # Check for duplicate declarations, then add the declaration and
     # bump the lastNum counter if necessary.
@@ -188,8 +148,6 @@ proc genStubs::declare {args} {
 	    puts stderr "Duplicate entry: declare $args"
 	}
     }
-    regsub -all const $decl CONST decl
-    regsub -all _XCONST $decl _Xconst decl
     regsub -all "\[ \t\n\]+" [string trim $decl] " " decl
     set decl [parseDecl $decl]
 
@@ -218,9 +176,14 @@ proc genStubs::declare {args} {
 #	None.
 
 proc genStubs::export {args} {
+    variable stubs
+    variable curName
+
     if {[llength $args] != 1} {
 	puts stderr "wrong # args: export $args"
     }
+    lassign $args decl
+
     return
 }
 
@@ -244,11 +207,10 @@ proc genStubs::rewriteFile {file text} {
     }
     set in [open ${file} r]
     set out [open ${file}.new w]
-    fconfigure $out -translation lf
 
     while {![eof $in]} {
 	set line [gets $in]
-	if {[regexp {!BEGIN!} $line]} {
+	if {[string match "*!BEGIN!*" $line]} {
 	    break
 	}
 	puts $out $line
@@ -257,7 +219,7 @@ proc genStubs::rewriteFile {file text} {
     puts $out $text
     while {![eof $in]} {
 	set line [gets $in]
-	if {[regexp {!END!} $line]} {
+	if {[string match "*!END!*" $line]} {
 	    break
 	}
     }
@@ -279,54 +241,42 @@ proc genStubs::rewriteFile {file text} {
 # Results:
 #	Returns the original text inside an appropriate #ifdef.
 
-proc genStubs::addPlatformGuard {plat iftxt {eltxt {}} {withCygwin 0}} {
+proc genStubs::addPlatformGuard {plat iftxt {eltxt {}}} {
     set text ""
     switch $plat {
 	win {
-	    append text "#if defined(__WIN32__)"
-	    if {$withCygwin} {
-		append text " || defined(__CYGWIN__)"
-	    }
-	    append text " /* WIN */\n${iftxt}"
-	    if {$eltxt != ""} {
+	    append text "#ifdef __WIN32__ /* WIN */\n${iftxt}"
+	    if {$eltxt ne ""} {
 		append text "#else /* WIN */\n${eltxt}"
 	    }
 	    append text "#endif /* WIN */\n"
 	}
 	unix {
-	    append text "#if !defined(__WIN32__)"
-	    if {$withCygwin} {
-		append text " && !defined(__CYGWIN__)"
-	    }
-	    append text " && !defined(MAC_OSX_TCL)\
+	    append text "#if !defined(__WIN32__) && !defined(MAC_OSX_TCL)\
 		    /* UNIX */\n${iftxt}"
-	    if {$eltxt != ""} {
+	    if {$eltxt ne ""} {
 		append text "#else /* UNIX */\n${eltxt}"
 	    }
 	    append text "#endif /* UNIX */\n"
 	}
 	macosx {
 	    append text "#ifdef MAC_OSX_TCL /* MACOSX */\n${iftxt}"
-	    if {$eltxt != ""} {
+	    if {$eltxt ne ""} {
 		append text "#else /* MACOSX */\n${eltxt}"
 	    }
 	    append text "#endif /* MACOSX */\n"
 	}
 	aqua {
 	    append text "#ifdef MAC_OSX_TK /* AQUA */\n${iftxt}"
-	    if {$eltxt != ""} {
+	    if {$eltxt ne ""} {
 		append text "#else /* AQUA */\n${eltxt}"
 	    }
 	    append text "#endif /* AQUA */\n"
 	}
 	x11 {
-	    append text "#if !(defined(__WIN32__)"
-	    if {$withCygwin} {
-		append text " || defined(__CYGWIN__)"
-	    }
-	    append text " || defined(MAC_OSX_TK))\
+	    append text "#if !(defined(__WIN32__) || defined(MAC_OSX_TK))\
 		    /* X11 */\n${iftxt}"
-	    if {$eltxt != ""} {
+	    if {$eltxt ne ""} {
 		append text "#else /* X11 */\n${eltxt}"
 	    }
 	    append text "#endif /* X11 */\n"
@@ -352,9 +302,10 @@ proc genStubs::addPlatformGuard {plat iftxt {eltxt {}} {withCygwin 0}} {
 #	None.
 
 proc genStubs::emitSlots {name textVar} {
+    variable stubs
     upvar $textVar text
 
-    forAllStubs $name makeSlot 1 text {"    VOID *reserved$i;\n"}
+    forAllStubs $name makeSlot 1 text {"    void *reserved$i;\n"}
     return
 }
 
@@ -456,14 +407,10 @@ proc genStubs::parseArg {arg} {
 #	Returns the formatted declaration string.
 
 proc genStubs::makeDecl {name decl index} {
-    variable scspec
     lassign $decl rtype fname args
 
     append text "/* $index */\n"
-    if {($rtype != "void") && ($rtype != "pascal void")} {
-	regsub -all void $rtype VOID rtype
-    }
-    set line "$scspec $rtype"
+    set line "EXTERN $rtype"
     set count [expr {2 - ([string length $line] / 8)}]
     append line [string range "\t\t\t" 0 $count]
     set pad [expr {24 - [string length $line]}]
@@ -477,31 +424,41 @@ proc genStubs::makeDecl {name decl index} {
 	append text ";\n"
 	return $text
     }
-    append line "$fname _ANSI_ARGS_("
+    append line "$fname "
 
-    regsub -all void $args VOID args
     set arg1 [lindex $args 0]
     switch -exact $arg1 {
-	VOID {
+	void {
 	    append line "(void)"
 	}
 	TCL_VARARGS {
-	    set arg [lindex $args 1]
-	    append line "TCL_VARARGS([lindex $arg 0],[lindex $arg 1])"
+	    set sep "("
+	    foreach arg [lrange $args 1 end] {
+		append line $sep
+		set next {}
+		append next [lindex $arg 0] " " [lindex $arg 1] \
+			[lindex $arg 2]
+		if {[string length $line] + [string length $next] \
+			+ $pad > 76} {
+		    append text $line \n
+		    set line "\t\t\t\t"
+		    set pad 28
+		}
+		append line $next
+		set sep ", "
+	    }
+	    append line ", ...)"
 	}
 	default {
 	    set sep "("
 	    foreach arg $args {
 		append line $sep
 		set next {}
-		append next [lindex $arg 0]
-		if {[string index $next end] != "*"} {
-		    append next " "
-		}
-		append next [lindex $arg 1] [lindex $arg 2]
+		append next [lindex $arg 0] " " [lindex $arg 1] \
+			[lindex $arg 2]
 		if {[string length $line] + [string length $next] \
 			+ $pad > 76} {
-		    append text [string trimright $line] \n
+		    append text $line \n
 		    set line "\t\t\t\t"
 		    set pad 28
 		}
@@ -511,7 +468,9 @@ proc genStubs::makeDecl {name decl index} {
 	    append line ")"
 	}
     }
-    return "$text$line);\n"
+    append text $line ";"
+    format "#ifndef %s_TCL_DECLARED\n#define %s_TCL_DECLARED\n%s\n#endif\n" \
+	    $fname $fname $text
 }
 
 # genStubs::makeMacro --
@@ -533,28 +492,81 @@ proc genStubs::makeMacro {name decl index} {
     append lfname [string range $fname 1 end]
 
     set text "#ifndef $fname\n#define $fname"
-    set arg1 [lindex $args 0]
-    set argList ""
-    switch -exact $arg1 {
-	void {
-	    set argList "()"
-	}
-	TCL_VARARGS {
-	}
-	default {
-	    set sep "("
-	    foreach arg $args {
-		append argList $sep [lindex $arg 1]
-		set sep ", "
-	    }
-	    append argList ")"
-	}
+    if {$args == ""} {
+	append text " \\\n\t(*${name}StubsPtr->$lfname)"
+	append text " /* $index */\n#endif\n"
+	return $text
     }
     append text " \\\n\t(${name}StubsPtr->$lfname)"
     append text " /* $index */\n#endif\n"
     return $text
 }
 
+# genStubs::makeStub --
+#
+#	Emits a stub function definition.
+#
+# Arguments:
+#	name	The interface name.
+#	decl	The function declaration.
+#	index	The slot index for this function.
+#
+# Results:
+#	Returns the formatted stub function definition.
+
+proc genStubs::makeStub {name decl index} {
+    lassign $decl rtype fname args
+
+    set lfname [string tolower [string index $fname 0]]
+    append lfname [string range $fname 1 end]
+
+    append text "/* Slot $index */\n" $rtype "\n" $fname
+
+    set arg1 [lindex $args 0]
+
+    if {![string compare $arg1 "TCL_VARARGS"]} {
+	lassign [lindex $args 1] type argName
+	append text " ($type$argName, ...)\n\{\n"
+	append text "    " $type " var;\n    va_list argList;\n"
+	if {[string compare $rtype "void"]} {
+	    append text "    " $rtype " resultValue;\n"
+	}
+	append text "\n    var = (" $type ") (va_start(argList, " \
+		$argName "), " $argName ");\n\n    "
+	if {[string compare $rtype "void"]} {
+	    append text "resultValue = "
+	}
+	append text "(" $name "StubsPtr->" $lfname "VA)(var, argList);\n"
+	append text "    va_end(argList);\n"
+	if {[string compare $rtype "void"]} {
+	    append text "return resultValue;\n"
+	}
+	append text "\}\n\n"
+	return $text
+    }
+
+    if {![string compare $arg1 "void"]} {
+	set argList "()"
+	set argDecls ""
+    } else {
+	set argList ""
+	set sep "("
+	foreach arg $args {
+	    append argList $sep [lindex $arg 1]
+	    append argDecls "    " [lindex $arg 0] " " \
+		    [lindex $arg 1] [lindex $arg 2] ";\n"
+	    set sep ", "
+	}
+	append argList ")"
+    }
+    append text $argList "\n" $argDecls "{\n    "
+    if {[string compare $rtype "void"]} {
+	append text "return "
+    }
+    append text "(" $name "StubsPtr->" $lfname ")" $argList ";\n}\n\n"
+    return $text
+}
+
 # genStubs::makeSlot --
 #
 #	Generate the stub table entry for a function.
@@ -578,28 +590,18 @@ proc genStubs::makeSlot {name decl index} {
 	append text $rtype " *" $lfname "; /* $index */\n"
 	return $text
     }
-    if {($rtype != "void") && ($rtype != "pascal void")} {
-	regsub -all void $rtype VOID rtype
-    }
-    if {[string range $rtype end-8 end] == "__stdcall"} {
-	append text [string trim [string range $rtype 0 end-9]] " (__stdcall *" $lfname ") _ANSI_ARGS_("
-    } else {
-	append text $rtype " (*" $lfname ") _ANSI_ARGS_("
-    }
-    regsub -all void $args VOID args
+    append text $rtype " (*" $lfname ") "
+
     set arg1 [lindex $args 0]
     switch -exact $arg1 {
-	VOID {
+	void {
 	    append text "(void)"
 	}
 	TCL_VARARGS {
 	    set sep "("
 	    foreach arg [lrange $args 1 end] {
-		append text $sep [lindex $arg 0]
-		if {[string index $text end] != "*"} {
-		    append text " "
-		}
-		append text [lindex $arg 1] [lindex $arg 2]
+		append text $sep [lindex $arg 0] " " [lindex $arg 1] \
+			[lindex $arg 2]
 		set sep ", "
 	    }
 	    append text ", ...)"
@@ -607,18 +609,15 @@ proc genStubs::makeSlot {name decl index} {
 	default {
 	    set sep "("
 	    foreach arg $args {
-		append text $sep [lindex $arg 0]
-		if {[string index $text end] != "*"} {
-		    append text " "
-		}
-		append text [lindex $arg 1] [lindex $arg 2]
+		append text $sep [lindex $arg 0] " " [lindex $arg 1] \
+			[lindex $arg 2]
 		set sep ", "
 	    }
 	    append text ")"
 	}
     }
 
-    append text "); /* $index */\n"
+    append text "; /* $index */\n"
     return $text
 }
 
@@ -664,7 +663,7 @@ proc genStubs::makeInit {name decl index} {
 # Results:
 #	None.
 
-proc genStubs::forAllStubs {name slotProc onAll textVar
+proc genStubs::forAllStubs {name slotProc onAll textVar \
 	{skipString {"/* Slot $i is reserved */\n"}}} {
     variable stubs
     upvar $textVar text
@@ -810,7 +809,7 @@ proc genStubs::forAllStubs {name slotProc onAll textVar
 		    eval {append temp} $skipString
 		}
 	    }
-	    append text [addPlatformGuard $plat $temp {} true]
+	    append text [addPlatformGuard $plat $temp]
 	}
 	## win ##
 	if {$block(win)} {
@@ -824,10 +823,10 @@ proc genStubs::forAllStubs {name slotProc onAll textVar
 		    eval {append temp} $skipString
 		}
 	    }
-	    append text [addPlatformGuard $plat $temp {} true]
+	    append text [addPlatformGuard $plat $temp]
 	}
 	## macosx ##
-	if {($block(unix) || $block(macosx)) && !$block(aqua) && !$block(x11)} {
+	if {$block(macosx) && !$block(aqua) && !$block(x11)} {
 	    set temp {}
 	    set lastNum -1
 	    foreach plat {unix macosx} {
@@ -896,7 +895,7 @@ proc genStubs::forAllStubs {name slotProc onAll textVar
 			} else {
 			    eval {set etxt} $skipString
 			    append temp [addPlatformGuard $plat [$slotProc \
-				    $name $stubs($name,$plat,$i) $i] $etxt true]
+				    $name $stubs($name,$plat,$i) $i] $etxt]
 			}
 			set emit 1
 			break
@@ -906,7 +905,7 @@ proc genStubs::forAllStubs {name slotProc onAll textVar
 		    eval {append temp} $skipString
 		}
 	    }
-	    append text [addPlatformGuard x11 $temp {} true]
+	    append text [addPlatformGuard x11 $temp]
 	}
     }
 }
@@ -923,6 +922,7 @@ proc genStubs::forAllStubs {name slotProc onAll textVar
 #	None.
 
 proc genStubs::emitDeclarations {name textVar} {
+    variable stubs
     upvar $textVar text
 
     append text "\n/*\n * Exported function declarations:\n */\n\n"
@@ -942,6 +942,7 @@ proc genStubs::emitDeclarations {name textVar} {
 #	None.
 
 proc genStubs::emitMacros {name textVar} {
+    variable stubs
     variable libraryName
     upvar $textVar text
 
@@ -971,19 +972,10 @@ proc genStubs::emitMacros {name textVar} {
 proc genStubs::emitHeader {name} {
     variable outDir
     variable hooks
-    variable epoch
-    variable revision
 
     set capName [string toupper [string index $name 0]]
     append capName [string range $name 1 end]
 
-    if {$epoch != ""} {
-	set CAPName [string toupper $name]
-	append text "\n"
-	append text "#define ${CAPName}_STUBS_EPOCH $epoch\n"
-	append text "#define ${CAPName}_STUBS_REVISION $revision\n"
-    }
-
     emitDeclarations $name text
 
     if {[info exists hooks($name)]} {
@@ -997,17 +989,13 @@ proc genStubs::emitHeader {name} {
     }
     append text "\ntypedef struct ${capName}Stubs {\n"
     append text "    int magic;\n"
-    if {$epoch != ""} {
-	append text "    int epoch;\n"
-	append text "    int revision;\n"
-    }
     append text "    struct ${capName}StubHooks *hooks;\n\n"
 
     emitSlots $name text
 
-    append text "} ${capName}Stubs;\n\n"
+    append text "} ${capName}Stubs;\n"
 
-    append text "#ifdef __cplusplus\nextern \"C\" {\n#endif\n"
+    append text "\n#ifdef __cplusplus\nextern \"C\" {\n#endif\n"
     append text "extern ${capName}Stubs *${name}StubsPtr;\n"
     append text "#ifdef __cplusplus\n}\n#endif\n"
 
@@ -1017,6 +1005,27 @@ proc genStubs::emitHeader {name} {
     return
 }
 
+# genStubs::emitStubs --
+#
+#	This function emits the body of the <name>Stubs.c file for
+#	the specified interface.
+#
+# Arguments:
+#	name	The name of the interface being emitted.
+#
+# Results:
+#	None.
+
+proc genStubs::emitStubs {name} {
+    variable outDir
+
+    append text "\n/*\n * Exported stub functions:\n */\n\n"
+    forAllStubs $name makeStub 0 text
+
+    rewriteFile [file join $outDir ${name}Stubs.c] $text
+    return
+}
+
 # genStubs::emitInit --
 #
 #	Generate the table initializers for an interface.
@@ -1029,8 +1038,8 @@ proc genStubs::emitHeader {name} {
 #	Returns the formatted output.
 
 proc genStubs::emitInit {name textVar} {
+    variable stubs
     variable hooks
-    variable epoch
     upvar $textVar text
 
     set capName [string toupper [string index $name 0]]
@@ -1047,11 +1056,6 @@ proc genStubs::emitInit {name textVar} {
     }
     append text "\n${capName}Stubs ${name}Stubs = \{\n"
     append text "    TCL_STUB_MAGIC,\n"
-    if {$epoch != ""} {
-	set CAPName [string toupper $name]
-	append text "    ${CAPName}_STUBS_EPOCH,\n"
-	append text "    ${CAPName}_STUBS_REVISION,\n"
-    }
     if {[info exists hooks($name)]} {
 	append text "    &${name}StubHooks,\n"
     } else {
@@ -1151,7 +1155,7 @@ proc genStubs::init {} {
 if {[string length [namespace which lassign]] == 0} {
     proc lassign {valueList args} {
 	if {[llength $args] == 0} {
-	    error "wrong # args: should be \"lassign list varName ?varName ...?\""
+	    error "wrong # args: lassign list varname ?varname..?"
 	}
 	uplevel [list foreach $args $valueList {break}]
 	return [lrange $valueList [llength $args] end]
