@@ -8,12 +8,14 @@
  *
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
+ *
+ * RCS: @(#) $Id: tclWinDde.c,v 1.31 2006/09/26 00:05:03 patthoyts Exp $
  */
 
 #include "tclInt.h"
-#include "tclPort.h"
 #include <dde.h>
 #include <ddeml.h>
+#include <tchar.h>
 
 /*
  * TCL_STORAGE_CLASS is set unconditionally to DLLEXPORT because the Dde_Init
@@ -79,14 +81,11 @@ static DWORD ddeInstance;	/* The application instance handle given to us
 				 * by DdeInitialize. */
 static int ddeIsServer = 0;
 
+#define TCL_DDE_VERSION		"1.3.2"
 #define TCL_DDE_PACKAGE_NAME	"dde"
 #define TCL_DDE_SERVICE_NAME	"TclEval"
 #define TCL_DDE_EXECUTE_RESULT	"$TCLEVAL$EXECUTE$RESULT"
 
-#define DDE_FLAG_ASYNC 1
-#define DDE_FLAG_BINARY 2
-#define DDE_FLAG_FORCE 4
-
 TCL_DECLARE_MUTEX(ddeMutex)
 
 /*
@@ -96,11 +95,10 @@ TCL_DECLARE_MUTEX(ddeMutex)
 static LRESULT CALLBACK	DdeClientWindowProc(HWND hwnd, UINT uMsg,
 			    WPARAM wParam, LPARAM lParam);
 static int		DdeCreateClient(struct DdeEnumServices *es);
-static BOOL CALLBACK	DdeEnumWindowsCallback(HWND hwndTarget,
-			    LPARAM lParam);
+static BOOL CALLBACK	DdeEnumWindowsCallback(HWND hwndTarget, LPARAM lParam);
 static void		DdeExitProc(ClientData clientData);
 static int		DdeGetServicesList(Tcl_Interp *interp,
-			    const char *serviceName, const char *topicName);
+			    char *serviceName, char *topicName);
 static HDDEDATA CALLBACK DdeServerProc(UINT uType, UINT uFmt, HCONV hConv,
 			    HSZ ddeTopic, HSZ ddeItem, HDDEDATA hData,
 			    DWORD dwData1, DWORD dwData2);
@@ -109,12 +107,13 @@ static LRESULT		DdeServicesOnAck(HWND hwnd, WPARAM wParam,
 static void		DeleteProc(ClientData clientData);
 static Tcl_Obj *	ExecuteRemoteObject(RegisteredInterp *riPtr,
 			    Tcl_Obj *ddeObjectPtr);
-static int		MakeDdeConnection(Tcl_Interp *interp,
-			    const char *name, HCONV *ddeConvPtr);
+static int		MakeDdeConnection(Tcl_Interp *interp, char *name,
+			    HCONV *ddeConvPtr);
 static void		SetDdeError(Tcl_Interp *interp);
-static int		DdeObjCmd(ClientData clientData,
+
+int			Tcl_DdeObjCmd(ClientData clientData,
 			    Tcl_Interp *interp, int objc,
-			    Tcl_Obj *const objv[]);
+			    Tcl_Obj *CONST objv[]);
 
 EXTERN int		Dde_Init(Tcl_Interp *interp);
 EXTERN int		Dde_SafeInit(Tcl_Interp *interp);
@@ -139,13 +138,16 @@ int
 Dde_Init(
     Tcl_Interp *interp)
 {
-    if (!Tcl_InitStubs(interp, "8.1", 0)) {
+    ThreadSpecificData *tsdPtr;
+
+    if (!Tcl_InitStubs(interp, "8.0", 0)) {
 	return TCL_ERROR;
     }
 
-    Tcl_CreateObjCommand(interp, "dde", DdeObjCmd, NULL, NULL);
+    Tcl_CreateObjCommand(interp, "dde", Tcl_DdeObjCmd, NULL, NULL);
+    tsdPtr = TCL_TSD_INIT(&dataKey);
     Tcl_CreateExitHandler(DdeExitProc, NULL);
-    return Tcl_PkgProvide(interp, TCL_DDE_PACKAGE_NAME, "1.3.3");
+    return Tcl_PkgProvide(interp, TCL_DDE_PACKAGE_NAME, TCL_DDE_VERSION);
 }
 
 /*
@@ -215,7 +217,7 @@ Initialize(void)
     if (ddeInstance == 0) {
 	Tcl_MutexLock(&ddeMutex);
 	if (ddeInstance == 0) {
-	    if (DdeInitialize(&ddeInstance, (PFNCALLBACK) DdeServerProc,
+	    if (DdeInitialize(&ddeInstance, DdeServerProc,
 		    CBF_SKIP_REGISTRATIONS | CBF_SKIP_UNREGISTRATIONS
 		    | CBF_FAIL_POKES, 0) != DMLERR_NO_ERROR) {
 		ddeInstance = 0;
@@ -263,20 +265,20 @@ Initialize(void)
  *----------------------------------------------------------------------
  */
 
-static const char *
+static char *
 DdeSetServerName(
     Tcl_Interp *interp,
-    const char *name, /* The name that will be used to refer to the
+    char *name,			/* The name that will be used to refer to the
 				 * interpreter in later "send" commands. Must
 				 * be globally unique. */
-    int flags,		/* DDE_FLAG_FORCE or 0 */
+    int exactName,		/* Should we make a unique name? 0 = unique */
     Tcl_Obj *handlerPtr)	/* Name of the optional proc/command to handle
 				 * incoming Dde eval's */
 {
     int suffix, offset;
     RegisteredInterp *riPtr, *prevPtr;
     Tcl_DString dString;
-    const char *actualName;
+    char *actualName;
     Tcl_Obj *srvListPtr = NULL, **srvPtrPtr = NULL;
     int n, srvCount = 0, lastSuffix, r = TCL_OK;
     ThreadSpecificData *tsdPtr = TCL_TSD_INIT(&dataKey);
@@ -325,7 +327,7 @@ DdeSetServerName(
     Tcl_DStringInit(&dString);
     actualName = name;
 
-    if (!(flags & DDE_FLAG_FORCE)) {
+    if (!exactName) {
 	r = DdeGetServicesList(interp, TCL_DDE_SERVICE_NAME, NULL);
 	if (r == TCL_OK) {
 	    srvListPtr = Tcl_GetObjResult(interp);
@@ -398,7 +400,7 @@ DdeSetServerName(
 	Tcl_ExposeCommand(interp, "dde", "dde");
     }
 
-    Tcl_CreateObjCommand(interp, "dde", DdeObjCmd,
+    Tcl_CreateObjCommand(interp, "dde", Tcl_DdeObjCmd,
 	    (ClientData) riPtr, DeleteProc);
     if (Tcl_IsSafe(interp)) {
 	Tcl_HideCommand(interp, "dde", "dde");
@@ -472,7 +474,7 @@ DeleteProc(
     ThreadSpecificData *tsdPtr = TCL_TSD_INIT(&dataKey);
 
     for (searchPtr = tsdPtr->interpListPtr, prevPtr = NULL;
-	    (searchPtr != NULL) && (searchPtr != riPtr);
+	    searchPtr != NULL && searchPtr != riPtr;
 	    prevPtr = searchPtr, searchPtr = searchPtr->nextPtr) {
 	/*
 	 * Empty loop body.
@@ -538,8 +540,7 @@ ExecuteRemoteObject(
 
 	Tcl_Obj *cmdPtr = Tcl_DuplicateObj(riPtr->handlerPtr);
 
-	result = Tcl_ListObjAppendElement(riPtr->interp, cmdPtr,
-		ddeObjectPtr);
+	result = Tcl_ListObjAppendElement(riPtr->interp, cmdPtr, ddeObjectPtr);
 	if (result == TCL_OK) {
 	    ddeObjectPtr = cmdPtr;
 	}
@@ -551,8 +552,7 @@ ExecuteRemoteObject(
 
     returnPackagePtr = Tcl_NewListObj(0, NULL);
 
-    Tcl_ListObjAppendElement(NULL, returnPackagePtr,
-	    Tcl_NewIntObj(result));
+    Tcl_ListObjAppendElement(NULL, returnPackagePtr, Tcl_NewIntObj(result));
     Tcl_ListObjAppendElement(NULL, returnPackagePtr,
 	    Tcl_GetObjResult(riPtr->interp));
 
@@ -595,7 +595,7 @@ static HDDEDATA CALLBACK
 DdeServerProc(
     UINT uType,			/* The type of DDE transaction we are
 				 * performing. */
-    UINT uFmt,			/* The format that data is sent or received */
+    UINT uFmt,			/* The format that data is sent or received. */
     HCONV hConv,		/* The conversation associated with the
 				 * current transaction. */
     HSZ ddeTopic, HSZ ddeItem,	/* String handles. Transaction-type
@@ -698,7 +698,7 @@ DdeServerProc(
 	 * execute.
 	 */
 
-	if ((uFmt != CF_TEXT) && (uFmt != CF_UNICODETEXT)) {
+	if (uFmt != CF_TEXT) {
 	    return (HDDEDATA) FALSE;
 	}
 
@@ -720,16 +720,10 @@ DdeServerProc(
 	    DdeQueryString(ddeInstance, ddeItem, utilString, (DWORD) len + 1,
 		    CP_WINANSI);
 	    if (stricmp(utilString, TCL_DDE_EXECUTE_RESULT) == 0) {
-		if (uFmt == CF_TEXT) {
-		    returnString =
-			    Tcl_GetStringFromObj(convPtr->returnPackagePtr, &len);
-		} else {
-		    returnString = (char *)
-			    Tcl_GetUnicodeFromObj(convPtr->returnPackagePtr, &len);
-		    len = 2 * len + 1;
-		}
-		ddeReturn = DdeCreateDataHandle(ddeInstance, (BYTE *)returnString,
-			(DWORD) len+1, 0, ddeItem, uFmt, 0);
+		returnString =
+			Tcl_GetStringFromObj(convPtr->returnPackagePtr, &len);
+		ddeReturn = DdeCreateDataHandle(ddeInstance, returnString,
+			(DWORD) len+1, 0, ddeItem, CF_TEXT, 0);
 	    } else {
 		if (Tcl_IsSafe(convPtr->riPtr->interp)) {
 		    ddeReturn = NULL;
@@ -738,17 +732,11 @@ DdeServerProc(
 			    convPtr->riPtr->interp, utilString, NULL,
 			    TCL_GLOBAL_ONLY);
 		    if (variableObjPtr != NULL) {
-			if (uFmt == CF_TEXT) {
-			    returnString = Tcl_GetStringFromObj(
-				    variableObjPtr, &len);
-			} else {
-			    returnString = (char *) Tcl_GetUnicodeFromObj(
-				    variableObjPtr, &len);
-			    len = 2 * len + 1;
-			}
+			returnString = Tcl_GetStringFromObj(variableObjPtr,
+				&len);
 			ddeReturn = DdeCreateDataHandle(ddeInstance,
-				(BYTE *)returnString, (DWORD) len+1, 0, ddeItem,
-				uFmt, 0);
+				returnString, (DWORD) len+1, 0, ddeItem,
+				CF_TEXT, 0);
 		    } else {
 			ddeReturn = NULL;
 		    }
@@ -765,7 +753,6 @@ DdeServerProc(
 	 */
 
 	Tcl_Obj *returnPackagePtr;
-	Tcl_UniChar *uniStr;
 
 	for (convPtr = tsdPtr->currentConversations; (convPtr != NULL)
 		&& (convPtr->hConv != hConv); convPtr = convPtr->nextPtr) {
@@ -779,21 +766,8 @@ DdeServerProc(
 	}
 
 	utilString = (char *) DdeAccessData(hData, &dlen);
-	uniStr = (Tcl_UniChar *) utilString;
-	if (!dlen) {
-	    /* Empty binary array. */
-	    ddeObjectPtr = Tcl_NewObj();
-	} else if ((dlen & 1) || uniStr[(dlen>>1)-1]) {
-	    /* Cannot be unicode, so assume utf-8 */
-	    if (!utilString[dlen-1]) {
-		dlen--;
-	    }
-	    ddeObjectPtr = Tcl_NewStringObj(utilString, dlen);
-	} else {
-	    /* unicode */
-	    dlen >>= 1;
-	    ddeObjectPtr = Tcl_NewUnicodeObj(uniStr, dlen - 1);
-	}
+	len = dlen;
+	ddeObjectPtr = Tcl_NewStringObj(utilString, -1);
 	Tcl_IncrRefCount(ddeObjectPtr);
 	DdeUnaccessData(hData);
 	if (convPtr->returnPackagePtr != NULL) {
@@ -905,7 +879,7 @@ DdeExitProc(
 static int
 MakeDdeConnection(
     Tcl_Interp *interp,		/* Used to report errors. */
-    const char *name,		/* The connection to use. */
+    char *name,			/* The connection to use. */
     HCONV *ddeConvPtr)
 {
     HSZ ddeTopic, ddeService;
@@ -981,6 +955,7 @@ DdeClientWindowProc(
     WPARAM wParam,
     LPARAM lParam)		/* (Potentially) our local handle */
 {
+
     switch (uMsg) {
     case WM_CREATE: {
 	LPCREATESTRUCT lpcs = (LPCREATESTRUCT) lParam;
@@ -988,14 +963,15 @@ DdeClientWindowProc(
 		(struct DdeEnumServices *) lpcs->lpCreateParams;
 
 #ifdef _WIN64
-	SetWindowLongPtr(hwnd, GWLP_USERDATA, (LONG_PTR) es);
+	SetWindowLongPtr(hwnd, GWLP_USERDATA, (long)es);
 #else
-	SetWindowLong(hwnd, GWL_USERDATA, (LONG) es);
+	SetWindowLong(hwnd, GWL_USERDATA, (long)es);
 #endif
 	return (LRESULT) 0L;
     }
     case WM_DDE_ACK:
 	return DdeServicesOnAck(hwnd, wParam, lParam);
+	break;
     default:
 	return DefWindowProc(hwnd, uMsg, wParam, lParam);
     }
@@ -1011,7 +987,7 @@ DdeServicesOnAck(
     ATOM service = (ATOM)LOWORD(lParam);
     ATOM topic = (ATOM)HIWORD(lParam);
     struct DdeEnumServices *es;
-    char sz[255];
+    TCHAR sz[255];
 
 #ifdef _WIN64
     es = (struct DdeEnumServices *) GetWindowLongPtr(hwnd, GWLP_USERDATA);
@@ -1019,8 +995,8 @@ DdeServicesOnAck(
     es = (struct DdeEnumServices *) GetWindowLong(hwnd, GWL_USERDATA);
 #endif
 
-    if ((es->service == (ATOM)0 || es->service == service)
-	    && (es->topic == (ATOM)0 || es->topic == topic)) {
+    if ((es->service == (ATOM)NULL || es->service == service)
+	    && (es->topic == (ATOM)NULL || es->topic == topic)) {
 	Tcl_Obj *matchPtr = Tcl_NewListObj(0, NULL);
 	Tcl_Obj *resultPtr = Tcl_GetObjResult(es->interp);
 
@@ -1062,7 +1038,7 @@ DdeEnumWindowsCallback(
     HWND hwndTarget,
     LPARAM lParam)
 {
-    DWORD_PTR dwResult = 0;
+    LRESULT dwResult = 0;
     struct DdeEnumServices *es = (struct DdeEnumServices *) lParam;
 
     SendMessageTimeout(hwndTarget, WM_DDE_INITIATE, (WPARAM)es->hwnd,
@@ -1074,16 +1050,16 @@ DdeEnumWindowsCallback(
 static int
 DdeGetServicesList(
     Tcl_Interp *interp,
-    const char *serviceName,
-    const char *topicName)
+    char *serviceName,
+    char *topicName)
 {
     struct DdeEnumServices es;
 
     es.interp = interp;
     es.result = TCL_OK;
     es.service = (serviceName == NULL)
-	    ? (ATOM)0 : GlobalAddAtom(serviceName);
-    es.topic = (topicName == NULL) ? (ATOM)0 : GlobalAddAtom(topicName);
+	    ? (ATOM)NULL : GlobalAddAtom(serviceName);
+    es.topic = (topicName == NULL) ? (ATOM)NULL : GlobalAddAtom(topicName);
 
     Tcl_ResetResult(interp); /* our list is to be appended to result. */
     DdeCreateClient(&es);
@@ -1092,10 +1068,10 @@ DdeGetServicesList(
     if (IsWindow(es.hwnd)) {
 	DestroyWindow(es.hwnd);
     }
-    if (es.service != (ATOM)0) {
+    if (es.service != (ATOM)NULL) {
 	GlobalDeleteAtom(es.service);
     }
-    if (es.topic != (ATOM)0) {
+    if (es.topic != (ATOM)NULL) {
 	GlobalDeleteAtom(es.topic);
     }
     return es.result;
@@ -1122,7 +1098,7 @@ static void
 SetDdeError(
     Tcl_Interp *interp)	    /* The interp to put the message in. */
 {
-    const char *errorMessage;
+    char *errorMessage;
 
     switch (DdeGetLastError(ddeInstance)) {
     case DMLERR_DATAACKTIMEOUT:
@@ -1146,7 +1122,7 @@ SetDdeError(
 /*
  *----------------------------------------------------------------------
  *
- * DdeObjCmd --
+ * Tcl_DdeObjCmd --
  *
  *	This function is invoked to process the "dde" Tcl command. See the
  *	user documentation for details on what it does.
@@ -1160,39 +1136,41 @@ SetDdeError(
  *----------------------------------------------------------------------
  */
 
-static int
-DdeObjCmd(
+int
+Tcl_DdeObjCmd(
     ClientData clientData,	/* Used only for deletion */
     Tcl_Interp *interp,		/* The interp we are sending from */
     int objc,			/* Number of arguments */
-    Tcl_Obj *const *objv)	/* The arguments */
+    Tcl_Obj *CONST * objv)	/* The arguments */
 {
-    static const char *ddeCommands[] = {
+    static CONST char *ddeCommands[] = {
 	"servername", "execute", "poke", "request", "services", "eval",
-	(char *) NULL};
+	(char *) NULL
+    };
     enum DdeSubcommands {
 	DDE_SERVERNAME, DDE_EXECUTE, DDE_POKE, DDE_REQUEST, DDE_SERVICES,
 	DDE_EVAL
     };
-    static const char *ddeSrvOptions[] = {
+    static CONST char *ddeSrvOptions[] = {
 	"-force", "-handler", "--", NULL
     };
     enum DdeSrvOptions {
 	DDE_SERVERNAME_EXACT, DDE_SERVERNAME_HANDLER, DDE_SERVERNAME_LAST,
     };
-    static const char *ddeExecOptions[] = {
+    static CONST char *ddeExecOptions[] = {
 	"-async", NULL
     };
-    static const char *ddeReqOptions[] = {
+    static CONST char *ddeReqOptions[] = {
 	"-binary", NULL
     };
 
-    int index, i, length, argIndex;
-    int flags = 0, result = TCL_OK, firstArg = 0;
+    int index, i, length;
+    int async = 0, binary = 0, exact = 0;
+    int result = TCL_OK, firstArg = 0;
     HSZ ddeService = NULL, ddeTopic = NULL, ddeItem = NULL, ddeCookie = NULL;
     HDDEDATA ddeData = NULL, ddeItemData = NULL, ddeReturn;
     HCONV hConv = NULL;
-    const char *serviceName = NULL, *topicName = NULL, *string;
+    char *serviceName = NULL, *topicName = NULL, *string;
     DWORD ddeResult;
     Tcl_Obj *objPtr, *handlerPtr = NULL;
 
@@ -1213,6 +1191,7 @@ DdeObjCmd(
     switch ((enum DdeSubcommands) index) {
     case DDE_SERVERNAME:
 	for (i = 2; i < objc; i++) {
+	    int argIndex;
 	    if (Tcl_GetIndexFromObj(interp, objv[i], ddeSrvOptions,
 		    "option", 0, &argIndex) != TCL_OK) {
 		/*
@@ -1227,7 +1206,7 @@ DdeObjCmd(
 		break;
 	    }
 	    if (argIndex == DDE_SERVERNAME_EXACT) {
-		flags |= DDE_FLAG_FORCE;
+		exact = 1;
 	    } else if (argIndex == DDE_SERVERNAME_HANDLER) {
 		if ((objc - i) == 1) {	/* return current handler */
 		    RegisteredInterp *riPtr = DdeGetRegistrationPtr(interp);
@@ -1260,9 +1239,10 @@ DdeObjCmd(
 	    firstArg = 2;
 	    break;
 	} else if (objc == 6) {
+	    int dummy;
 	    if (Tcl_GetIndexFromObj(NULL, objv[2], ddeExecOptions, "option", 0,
-		    &argIndex) == TCL_OK) {
-		flags |= DDE_FLAG_ASYNC;
+		    &dummy) == TCL_OK) {
+		async = 1;
 		firstArg = 3;
 		break;
 	    }
@@ -1287,7 +1267,7 @@ DdeObjCmd(
 	    int dummy;
 	    if (Tcl_GetIndexFromObj(NULL, objv[2], ddeReqOptions, "option", 0,
 		    &dummy) == TCL_OK) {
-		flags |= DDE_FLAG_BINARY;
+		binary = 1;
 		firstArg = 3;
 		break;
 	    }
@@ -1313,13 +1293,15 @@ DdeObjCmd(
 	    Tcl_WrongNumArgs(interp, 2, objv, "?-async? serviceName args");
 	    return TCL_ERROR;
 	} else {
+	    int dummy;
+
 	    firstArg = 2;
-	    if (Tcl_GetIndexFromObj(NULL, objv[2], ddeExecOptions, "option",
-		    0, &argIndex) == TCL_OK) {
+	    if (Tcl_GetIndexFromObj(NULL, objv[2], ddeExecOptions, "option", 0,
+		    &dummy) == TCL_OK) {
 		if (objc < 5) {
 		    goto wrongDdeEvalArgs;
 		}
-		flags |= DDE_FLAG_ASYNC;
+		async = 1;
 		firstArg++;
 	    }
 	    break;
@@ -1337,7 +1319,7 @@ DdeObjCmd(
     if (length == 0) {
 	serviceName = NULL;
     } else if ((index != DDE_SERVERNAME) && (index != DDE_EVAL)) {
-	ddeService = DdeCreateStringHandle(ddeInstance, (void *) serviceName,
+	ddeService = DdeCreateStringHandle(ddeInstance, serviceName,
 		CP_WINANSI);
     }
 
@@ -1346,15 +1328,14 @@ DdeObjCmd(
 	if (length == 0) {
 	    topicName = NULL;
 	} else {
-	    ddeTopic = DdeCreateStringHandle(ddeInstance, (void *) topicName,
+	    ddeTopic = DdeCreateStringHandle(ddeInstance, topicName,
 		    CP_WINANSI);
 	}
     }
 
     switch ((enum DdeSubcommands) index) {
     case DDE_SERVERNAME:
-	serviceName = DdeSetServerName(interp, serviceName, flags,
-		handlerPtr);
+	serviceName = DdeSetServerName(interp, serviceName, exact, handlerPtr);
 	if (serviceName != NULL) {
 	    Tcl_SetObjResult(interp, Tcl_NewStringObj(serviceName, -1));
 	} else {
@@ -1364,8 +1345,8 @@ DdeObjCmd(
 
     case DDE_EXECUTE: {
 	int dataLength;
-	BYTE *dataString = (BYTE *) Tcl_GetStringFromObj(
-		objv[firstArg + 2], &dataLength);
+	char *dataString = Tcl_GetStringFromObj(objv[firstArg + 2],
+		&dataLength);
 
 	if (dataLength == 0) {
 	    Tcl_SetObjResult(interp,
@@ -1386,7 +1367,7 @@ DdeObjCmd(
 	ddeData = DdeCreateDataHandle(ddeInstance, dataString,
 		(DWORD) dataLength+1, 0, 0, CF_TEXT, 0);
 	if (ddeData != NULL) {
-	    if (flags & DDE_FLAG_ASYNC) {
+	    if (async) {
 		DdeClientTransaction((LPBYTE) ddeData, 0xFFFFFFFF, hConv, 0,
 			CF_TEXT, XTYP_EXECUTE, TIMEOUT_ASYNC, &ddeResult);
 		DdeAbandonTransaction(ddeInstance, hConv, ddeResult);
@@ -1406,8 +1387,7 @@ DdeObjCmd(
 	break;
     }
     case DDE_REQUEST: {
-	const char *itemString = Tcl_GetStringFromObj(objv[firstArg + 2],
-		&length);
+	char *itemString = Tcl_GetStringFromObj(objv[firstArg + 2], &length);
 
 	if (length == 0) {
 	    Tcl_SetObjResult(interp,
@@ -1424,7 +1404,7 @@ DdeObjCmd(
 	    result = TCL_ERROR;
 	} else {
 	    Tcl_Obj *returnObjPtr;
-	    ddeItem = DdeCreateStringHandle(ddeInstance, (void *)itemString,
+	    ddeItem = DdeCreateStringHandle(ddeInstance, itemString,
 		    CP_WINANSI);
 	    if (ddeItem != NULL) {
 		ddeData = DdeClientTransaction(NULL, 0, hConv, ddeItem,
@@ -1434,17 +1414,13 @@ DdeObjCmd(
 		    result = TCL_ERROR;
 		} else {
 		    DWORD tmp;
-		    const char *dataString = (const char *) DdeAccessData(ddeData, &tmp);
+		    char *dataString = DdeAccessData(ddeData, &tmp);
 
-		    if (flags & DDE_FLAG_BINARY) {
-			returnObjPtr =
-				Tcl_NewByteArrayObj((BYTE *) dataString, (int) tmp);
-		    } else {
-			if (tmp && !dataString[tmp-1]) {
-			    --tmp;
-			}
-			returnObjPtr = Tcl_NewStringObj(dataString,
+		    if (binary) {
+			returnObjPtr = Tcl_NewByteArrayObj(dataString,
 				(int) tmp);
+		    } else {
+			returnObjPtr = Tcl_NewStringObj(dataString, -1);
 		    }
 		    DdeUnaccessData(ddeData);
 		    DdeFreeDataHandle(ddeData);
@@ -1459,9 +1435,8 @@ DdeObjCmd(
 	break;
     }
     case DDE_POKE: {
-	const char *itemString = Tcl_GetStringFromObj(objv[firstArg + 2],
-		&length);
-	BYTE *dataString;
+	char *itemString = Tcl_GetStringFromObj(objv[firstArg + 2], &length);
+	char *dataString;
 
 	if (length == 0) {
 	    Tcl_SetObjResult(interp,
@@ -1469,8 +1444,7 @@ DdeObjCmd(
 	    result = TCL_ERROR;
 	    goto cleanup;
 	}
-	dataString = (BYTE *) Tcl_GetStringFromObj(objv[firstArg + 3],
-		&length);
+	dataString = Tcl_GetStringFromObj(objv[firstArg + 3], &length);
 
 	hConv = DdeConnect(ddeInstance, ddeService, ddeTopic, NULL);
 	DdeFreeStringHandle(ddeInstance, ddeService);
@@ -1480,7 +1454,7 @@ DdeObjCmd(
 	    SetDdeError(interp);
 	    result = TCL_ERROR;
 	} else {
-	    ddeItem = DdeCreateStringHandle(ddeInstance, (void *) itemString,
+	    ddeItem = DdeCreateStringHandle(ddeInstance, itemString,
 		    CP_WINANSI);
 	    if (ddeItem != NULL) {
 		ddeData = DdeClientTransaction(dataString, (DWORD) length+1,
@@ -1512,8 +1486,8 @@ DdeObjCmd(
 	    goto cleanup;
 	}
 
-	objc -= firstArg + 1;
-	objv += firstArg + 1;
+	objc -= (async + 3);
+	objv += (async + 3);
 
 	/*
 	 * See if the target interpreter is local. If so, execute the command
@@ -1625,10 +1599,10 @@ DdeObjCmd(
 
 	    objPtr = Tcl_ConcatObj(objc, objv);
 	    string = Tcl_GetStringFromObj(objPtr, &length);
-	    ddeItemData = DdeCreateDataHandle(ddeInstance,
-		    (BYTE *) string, (DWORD) length+1, 0, 0, CF_TEXT, 0);
+	    ddeItemData = DdeCreateDataHandle(ddeInstance, string,
+		    (DWORD) length+1, 0, 0, CF_TEXT, 0);
 
-	    if (flags & DDE_FLAG_ASYNC) {
+	    if (async) {
 		ddeData = DdeClientTransaction((LPBYTE) ddeItemData,
 			0xFFFFFFFF, hConv, 0,
 			CF_TEXT, XTYP_EXECUTE, TIMEOUT_ASYNC, &ddeResult);
@@ -1652,7 +1626,7 @@ DdeObjCmd(
 		result = TCL_ERROR;
 	    }
 
-	    if (!(flags & DDE_FLAG_ASYNC)) {
+	    if (async == 0) {
 		Tcl_Obj *resultPtr;
 
 		/*
@@ -1668,7 +1642,7 @@ DdeObjCmd(
 		length = DdeGetData(ddeData, NULL, 0, 0);
 		Tcl_SetObjLength(resultPtr, length);
 		string = Tcl_GetString(resultPtr);
-		DdeGetData(ddeData, (BYTE *) string, (DWORD) length, 0);
+		DdeGetData(ddeData, string, (DWORD) length, 0);
 		Tcl_SetObjLength(resultPtr, (int) strlen(string));
 
 		if (Tcl_ListObjIndex(NULL, resultPtr, 0, &objPtr) != TCL_OK) {
